/*
 Copyright 2015 Nicolas Melot

 This file is part of Drake-FFT2_streaming.

 Drake-FFT2_streaming is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Drake-FFT2_streaming is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Drake-FFT2_streaming. If not, see <http://www.gnu.org/licenses/>.

*/


#include <stdlib.h>
#include <math.h>
#include <string.h>

#include <pelib/float.h>
#include <drake/schedule.h>
#include <drake/node.h>
#include <drake/link.h>
#include <drake/platform.h>
#include <drake/eval.h>
#include <math.h>

#include "args.h"

static int total_data;
static int n = 32;
static int work_unit = 64;

int
drake_init(task_t *task, void* arg)
{
	char *seekid = task->name;
	size_t index = 0;
	while(seekid[index] < '0' || seekid[index] > '9')
	{
		index++;
	}
	char* n_ptr = &seekid[index];
	size_t length = 0;
	while(seekid[index + length] != '_')
	{
		length++;
	}
	char* n_str = malloc(sizeof(char) * (length + 1));
	memcpy(n_str, n_ptr, length);
	n_str[length] = '\0';
	int divider = 1 << atoi(n_str);
	app_args_t args = parse_args((args_t*)arg);
	work_unit = n * (int)(1 << args.n);
	n = work_unit / divider;
	//printf("[%s:%s:%d][Task %u:%s] n = %d\n", __FILE__, __FUNCTION__, __LINE__, task->id, task->name, n);
	free(n_str);
	total_data = 2 * n;

	return 1;
}

int
drake_start(task_t *task)
{
	return 1;
}

static int sum = 0;
static int runtimes = 0;
int
drake_run(task_t *task)
{
	link_t *in, *out;
	in = pelib_map_read(string, link_tp)(pelib_map_find(string, link_tp)(task->pred, "input")).value;
	out = pelib_map_read(string, link_tp)(pelib_map_find(string, link_tp)(task->succ, "output")).value;

	if(pelib_cfifo_length(float)(((cfifo_t(float)*)in->buffer)) >= (2 * work_unit) && pelib_cfifo_capacity(float)(((cfifo_t(float)*)out->buffer)) - pelib_cfifo_length(float)(((cfifo_t(float)*)out->buffer)) >= (2 * work_unit))
	{

	int i, j;
	float val;
	for(j = 0; j < ((float)(2 * work_unit)) / (float)total_data; j++)
		{
			for (i = 0; i < total_data; i += 4)
			{
				val = pelib_cfifo_peek(float)(((cfifo_t(float)*)in->buffer), i);
				pelib_cfifo_push(float)(((cfifo_t(float)*)out->buffer), val);
				val = pelib_cfifo_peek(float)(((cfifo_t(float)*)in->buffer), i + 1);
				pelib_cfifo_push(float)(((cfifo_t(float)*)out->buffer), val);	       
			}

			for (i = 2; i < total_data; i += 4)
			{
				val = pelib_cfifo_peek(float)(((cfifo_t(float)*)in->buffer), i);
				pelib_cfifo_push(float)(((cfifo_t(float)*)out->buffer), val);
				val = pelib_cfifo_peek(float)(((cfifo_t(float)*)in->buffer), i + 1);
				pelib_cfifo_push(float)(((cfifo_t(float)*)out->buffer), val);
			}

			for(i = 0; i < total_data; i++)
			{
				val = pelib_cfifo_pop(float)(((cfifo_t(float)*)in->buffer));
			}
		}
	}

	int killed = ((in->prod == NULL) ? 1 : in->prod->status >= TASK_KILLED);
	int empty = pelib_cfifo_length(float)(((cfifo_t(float)*)in->buffer)) < (2 * work_unit);

	if(killed && empty)
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

int
drake_kill(task_t *task)
{
	return 0;
}

int
drake_destroy(task_t *task)
{
	return 0;
}
