/*
 Copyright 2015 Nicolas Melot

 This file is part of Drake-FFT2_streaming.

 Drake-FFT2_streaming is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Drake-FFT2_streaming is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Drake-FFT2_streaming. If not, see <http://www.gnu.org/licenses/>.

*/


#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <math.h>

#include <pelib/float.h>
#include <drake/schedule.h>
#include <drake/node.h>
#include <drake/link.h>
#include <drake/platform.h>
#include <drake/eval.h>

#include "args.h"

static int n = 32;
static int max_rounds = 1;
static float time = 0;

static drake_time_t start_time, max_time, round_time, period;

int
drake_init(task_t *task, void* arg)
{
	app_args_t args = parse_args((args_t*)arg);
	max_rounds = args.rounds;
	time = args.time;
	n = n * (int)(1 << args.n);
	//printf("[%s:%s:%d][Task %u:%s] n = %d\n", __FILE__, __FUNCTION__, __LINE__, task->id, task->name, n);
	//printf("[%s:%s:%d][Task %u:%s] time == %f\n", __FILE__, __FUNCTION__, __LINE__, task->id, task->name, time);

	return 1;
}

int drake_start(task_t *task)
{
	if(time > 0)
	{
		start_time = drake_platform_time_alloc();
		max_time = drake_platform_time_alloc();
		round_time = drake_platform_time_alloc();
		period = drake_platform_time_alloc();

		drake_platform_time_get(start_time);
		drake_platform_time_init(max_time, time);
	}
	return 1;
}

int drake_run(task_t *task)
{
	static int numel = 0;
	link_t *out = pelib_map_read(string, link_tp)(pelib_map_find(string, link_tp)(task->succ, "output")).value;
	
	if(pelib_cfifo_capacity(float)((cfifo_t(float)*)out->buffer) < 2 * n)
	{
		fprintf(stderr, "[%s:%s:%d][Task %d:%s] Not enough memory allocated\n", __FILE__, __FUNCTION__, __LINE__, task->id, task->name);
		return 1;
	}

	if(pelib_cfifo_capacity(float)((cfifo_t(float)*)out->buffer) - pelib_cfifo_length(float)((cfifo_t(float)*)out->buffer) < 2 * n)
	{
		// No room to produce more tokens try next time
		return 0;
	}

	numel++;
	int i, j, res;
	float val_i, val_r;
	val_r = 0;
	val_i = 0;
	res = pelib_cfifo_push(float)((cfifo_t(float)*)out->buffer, val_r);
	res = pelib_cfifo_push(float)((cfifo_t(float)*)out->buffer, val_i);
	val_r = 1;
	pelib_cfifo_push(float)((cfifo_t(float)*)out->buffer, val_r);
	pelib_cfifo_push(float)((cfifo_t(float)*)out->buffer, val_i);
	val_r = 0;

	for(j = 0; j < n - 2; j++)
	{
		pelib_cfifo_push(float)((cfifo_t(float)*)out->buffer, val_r);
		pelib_cfifo_push(float)((cfifo_t(float)*)out->buffer, val_i);
	}

	if(time > 0 && 1)
	{
		drake_platform_time_get(round_time);
		drake_platform_time_subtract(period, round_time, start_time);
	}

	if((max_rounds > 0 && numel >= max_rounds) || (time > 0 && drake_platform_time_greater(period, max_time)))
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

int
drake_kill(task_t *task)
{
	/* Do nothing */
	return 0;
}

int
drake_destroy(task_t *task)
{
	if(time > 0)
	{
		drake_platform_time_destroy(start_time);
		drake_platform_time_destroy(max_time);
		drake_platform_time_destroy(round_time);
		drake_platform_time_destroy(period);
	}

	/* Do nothing */
	return 0;
}

