/*
 Copyright 2015 Nicolas Melot

 This file is part of Drake-FFT2_streaming.

 Drake-FFT2_streaming is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Drake-FFT2_streaming is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Drake-FFT2_streaming. If not, see <http://www.gnu.org/licenses/>.

*/


#include <stdlib.h>
#include <math.h>
#include <string.h>

#include <pelib/float.h>
#include <drake/schedule.h>
#include <drake/node.h>
#include <drake/link.h>
#include <drake/platform.h>
#include <drake/eval.h>
#include <math.h>

#include "args.h"

enum turn_t {left, right};

static int n = 32;
static enum turn_t turn = left;

int
drake_init(task_t *task, void* arg)
{
	app_args_t args = parse_args((args_t*)arg);
	n = n * (int)(1 << args.n);
	//printf("[%s:%s:%d][Task %u:%s] n = %d\n", __FILE__, __FUNCTION__, __LINE__, task->id, task->name, n);

	return 1;
}

int
drake_start(task_t *task)
{
	return 1;
}

int
drake_run(task_t *task)
{
	link_t *in_left = pelib_map_read(string, link_tp)(pelib_map_find(string, link_tp)(task->pred, "left")).value;
	link_t *in_right = pelib_map_read(string, link_tp)(pelib_map_find(string, link_tp)(task->pred, "right")).value;
	link_t *in = pelib_map_read(string, link_tp)(pelib_map_find(string, link_tp)(task->pred, turn == left ? "left" : "right")).value;
	link_t *out = pelib_map_read(string, link_tp)(pelib_map_find(string, link_tp)(task->succ, "output")).value;

	if(pelib_cfifo_length(float)(((cfifo_t(float)*)in->buffer)) >= (2 * n) && pelib_cfifo_capacity(float)(((cfifo_t(float)*)out->buffer)) - pelib_cfifo_length(float)(((cfifo_t(float)*)out->buffer)) >= (2 * n))
	{
	float val, j;
		for(j = 0; j < (2 * n); j++)
		{
			val = pelib_cfifo_pop(float)(((cfifo_t(float)*)in->buffer));
			pelib_cfifo_push(float)(((cfifo_t(float)*)out->buffer), val);
		}
	}
	turn = (turn == left ? right : left);

	int killed_left = ((in_left->prod == NULL) ? 1 : in_left->prod->status >= TASK_KILLED);
	int killed_right = ((in_right->prod == NULL) ? 1 : in_right->prod->status >= TASK_KILLED);
	int empty_left = pelib_cfifo_length(float)(((cfifo_t(float)*)in_left->buffer)) == 0;
	int empty_right = pelib_cfifo_length(float)(((cfifo_t(float)*)in_right->buffer)) == 0;

	if(killed_left && killed_right && empty_left && empty_right)
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

int drake_kill(task_t *task)
{
	/* Do nothing */
	return 0;
}

int drake_destroy(task_t *task)
{
	/* Do nothing */
	return 0;
}

